package top.jerryyan.RN.A.VersionUpgrade;

import android.app.Activity;
import android.app.AlertDialog;
import android.content.DialogInterface;
import android.content.Intent;
import android.content.pm.PackageInfo;
import android.content.pm.PackageManager;
import android.net.Uri;
import android.os.Build;
import android.os.Handler;
import android.os.Message;
import android.provider.Settings;
import android.util.Log;

import com.facebook.react.bridge.ReactApplicationContext;
import com.facebook.react.bridge.ReactContextBaseJavaModule;
import com.facebook.react.bridge.ReactMethod;

import org.json.JSONException;
import org.json.JSONObject;

import java.io.IOException;
import java.util.HashMap;
import java.util.Map;

import okhttp3.Call;
import okhttp3.Callback;
import okhttp3.OkHttpClient;
import okhttp3.Request;
import okhttp3.Response;

public class UpgradeModule extends ReactContextBaseJavaModule {
    private final ReactApplicationContext reactContext;
    private String DOWNLOAD_URL = "http://luntan.qgmzbxs.com/source/tsgzCircles.apk";
    private String TITLE = "更新";
    private String CONTENT = "大家好，我是勤劳的催更新菌\n点击更新，让我们一起成为一天更新八次的暴躁老哥吧";

    public UpgradeModule(ReactApplicationContext reactContext) {
        super(reactContext);
        this.reactContext = reactContext;
    }

    @Override
    public String getName() {
        return "VersionUpgrade";
    }

    @Override
    public Map<String, Object> getConstants() {
        final Map<String, Object> constants = new HashMap<>();
        return constants;
    }

    private boolean _hasPermissionToInstall() {
        Activity activity = reactContext.getCurrentActivity();
        return activity != null && (Build.VERSION.SDK_INT < Build.VERSION_CODES.O || activity.getPackageManager().canRequestPackageInstalls());
    }

    private void _requestInstallPermission() {
        Activity activity = reactContext.getCurrentActivity();
        if (activity == null) return;
        if (Build.VERSION.SDK_INT >= Build.VERSION_CODES.O) {
            Uri packageURI = Uri.parse("package:" + activity.getPackageName());
            Intent intent = new Intent(Settings.ACTION_MANAGE_UNKNOWN_APP_SOURCES, packageURI);
            (activity).startActivityForResult(intent, 2);
        }
    }

    void _alertDialog(){
        final UpgradeModule that = this;
        AlertDialog dialog = new AlertDialog.Builder(reactContext.getCurrentActivity())
                .setTitle(TITLE)
                .setMessage(CONTENT)
                .setNeutralButton("取消",
                        new DialogInterface.OnClickListener() {
                            @Override
                            public void onClick(DialogInterface dialog, int which) {
                                dialog.dismiss();
                            }
                        }
                )
                .setNegativeButton("更新",
                        new DialogInterface.OnClickListener() {
                            @Override
                            public void onClick(DialogInterface dialog, int which) {
                                that._startDownload();
                            }
                        }
                )
                .show();
        dialog.setCanceledOnTouchOutside(true);//可选
        dialog.setCancelable(true);//可选
    }

    private void _startDownload(){

    }

    @ReactMethod
    public void checkUpgrade(){
        final UpgradeModule that = this;
        OkHttpClient okHttpClient = new OkHttpClient();
        String upgradeUrl = "https://www.tsgzvore.com/versionCheck";
        PackageManager manager = reactContext.getPackageManager();
        try {
            String packageName = reactContext.getPackageName();
            PackageInfo info = manager.getPackageInfo(packageName, 0);
            String versionName = info.versionName;
            upgradeUrl = upgradeUrl
                    .concat("?packageName=")
                    .concat(packageName)
                    .concat("&version=")
                    .concat(versionName);
        } catch (PackageManager.NameNotFoundException e) {
            Log.e("Version Upgrade", "NOT FOUND MY PACKAGE TAT", e);
            return;
        }
        final Request request = new Request.Builder()
                .url(upgradeUrl)
                .method("GET", null)
                .build();
        Call call = okHttpClient.newCall(request);
        final Handler handler = new ShowUpgradeAlertHandler(that);
        call.enqueue(new Callback() {
            @Override
            public void onFailure(Call call, IOException e) {
                Log.e("VERSION CHECK", call.toString(), e);
            }

            @Override
            public void onResponse(Call call, Response response) {
                if(response.isSuccessful()){
                    if(response.body() == null) return;
                    try {
                        String jsonText = response.body().string();
                        Log.d("VERSION CHECK", jsonText);
                        JSONObject json = new JSONObject(jsonText);
                        int status = json.getInt("status");
                        if(status > 0){
                            CONTENT = json.getString("message");
//                            String version = json.getString("version");
                            JSONObject jsonData = json.getJSONObject("data");
                            DOWNLOAD_URL = jsonData.getString("url");
                            handler.sendEmptyMessage(0);
                        }
                    } catch (IOException e) {
                        Log.e("REQUEST FAILED", "AAA", e);
                    } catch (JSONException e) {
                        Log.e("JSON FAILED", "BBB", e);
                    }
                }
            }
        });
    }
}

final class ShowUpgradeAlertHandler extends Handler
{
    private UpgradeModule module;

    ShowUpgradeAlertHandler(UpgradeModule module)
    {
        this.module = module;
    }
    @Override
    public void handleMessage(Message message){
        module._alertDialog();
    }
}